/*
 * ATMGUI.java
 * 
 * $Id: ATMGUI.java,v 1.6 2012-05-16 06:18:09 jbf9160 Exp $
 * 
 * $Log: ATMGUI.java,v $
 * Revision 1.6  2012-05-16 06:18:09  jbf9160
 * Comments added.
 *
 * Revision 1.5  2012-05-15 11:12:10  jbf9160
 * Small updates.
 * MVC Fully working.
 *
 * Revision 1.4  2012-05-15 10:52:05  jbf9160
 * Interface fully working.
 *
 * Revision 1.3  2012-05-15 06:48:25  jbf9160
 * Added headers... hopefully not to late.
 *
 */

import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * ATMGUI is the view and controller for the ATM model
 * 
 * @author James Fator (jbf9160@rit.edu)
 */
public class ATMGUI extends JFrame {
	
	private static final long serialVersionUID = 1L;
	private ATMPhases model;
	private Terminal screen;
	private BankGUI obs;
	
	/**
	 * Constructor for the ATMGUI class.
	 * Creates the window and all internal components
	 * 
	 * @param obs the BankGui that initialized this class
	 * @param num the number of the window
	 */
	public ATMGUI(BankGUI obs, int num) {
		this.obs = obs;
		screen = new Terminal();
		this.model = new ATMPhases(this, screen);
		JPanel main = new JPanel(new GridLayout(1,2));
		
		//Add Terminal to the window
		main.add(screen);
		
		JPanel keypad = new JPanel(new GridLayout(4,4));
		for (int i = 0; i < 16; i++) {
			JButton temp;
			JPanel test = null;
			if (i == 0) {
				temp = new JButton("< 1");
				test = new JPanel(new FlowLayout(0,-2,12));
			} else if (i == 4) {
				temp = new JButton("< 2");
				test = new JPanel(new FlowLayout(0,-2,2));
			} else if (i < 4) {
				temp = new JButton(Integer.toString(i));
			} else if (i == 8) {
				temp = new JButton("< 3");
				test = new JPanel(new FlowLayout(0,-2,12));
			} else if (i < 8) {
				temp = new JButton(Integer.toString(i-1));
			} else if (i == 12) {
				temp = new JButton("< 4");
				test = new JPanel(new FlowLayout(0,-2,2));
			} else if (i < 12) {
				temp = new JButton(Integer.toString(i-2));
			} else if (i == 13) {
				temp = new JButton("CLOSE");
			} else if (i == 14) {
				temp = new JButton(Integer.toString(0));
			} else {
				temp = new JButton("OKAY");
			}
			temp.addActionListener(new KeypadListener());
			if (i == 0 || i == 4 || i == 8 || i == 12) {
				test.add(temp);
				keypad.add(test);
			} else {
				keypad.add(temp);
			}
		}
		//Add number buttons and option buttons to window
		main.add(keypad);
		
		this.add(main);
		this.setSize(560, 190);
		this.setResizable(false);
		this.setTitle("James Fator " + Integer.toString(num));
		this.setLocationRelativeTo(null);
		this.setVisible(true);
		
		//Let model know everything is up and running
		model.display();
	}
	
	/**
	 * validId checks with the BankGUI if the id is valid
	 * @param id of Account to check
	 * @return true if Account with id exists
	 */
	/*Package*/ boolean validId(int id) {
		return obs.validId(id);
	}
	
	/**
	 * validPin checks with the BankGUI if 
	 * the pin works with the given Account
	 * 
	 * @param id of Account to check
	 * @param pin to test with Account
	 * @return true if pin matches Account
	 * 
	 * @assume id is valid
	 */
	/*Package*/ boolean validPin(int id, int pin) {
		return obs.validPin(id, pin);
	}
	
	/**
	 * getBal retrieves the balance of the account from the
	 * BankGUI
	 * 
	 * @param id of Account
	 * @param pin of Account
	 * @return amount in Account
	 * 
	 * @assume id and pin are valid
	 */
	/*Package*/ double getBal(int id, int pin) {
		return obs.getBal(id, pin);
	}
	
	/**
	 * deposit adds money to Account
	 * 
	 * @param id of Account
	 * @param pin of Account
	 * @param bal amount to be added
	 * 
	 * @assume id and pin are valid
	 */
	/*Package*/ void deposit(int id, int pin, double bal) {
		obs.deposit(id, pin, bal);
	}
	
	/**
	 * withdraw removes money from account if the account
	 * has enough
	 * 
	 * @param id of Account
	 * @param pin of Account
	 * @param bal amount to be removed
	 * @return true if withdraw was successful
	 * 
	 * @assume id and pin are valid
	 */
	/*Package*/ boolean withdraw(int id, int pin, double bal) {
		return obs.withdraw(id, pin, bal);
	}
	
	/**
	 * KeypadListener is an ActonListener for the ATMGUI class
	 * It informs the ATM model when a button has been pressed.
	 */
	class KeypadListener implements ActionListener {
		
		/**
		 * actionPerformed notifies the ATM model when
		 * a button has been clicked.
		 */
		@Override
		public void actionPerformed(ActionEvent arg0) {
			String button = arg0.getActionCommand();
			if (button.equals("OKAY")) {
				model.okay();
			} else if (button.equals("CLOSE")) {
				model.close();
			} else if (button.equals("< 1")) {
				model.one();
			} else if (button.equals("< 2")) {
				model.two();
			} else if (button.equals("< 3")) {
				model.three();
			} else if (button.equals("< 4")) {
				model.four();
			} else {
				model.number(Integer.parseInt(button));
			}
		}
	}
}