/*
 * ATMPhases.java
 * 
 * $Id: ATMPhases.java,v 1.3 2012-05-16 06:18:10 jbf9160 Exp $
 * 
 * $Log: ATMPhases.java,v $
 * Revision 1.3  2012-05-16 06:18:10  jbf9160
 * Comments added.
 *
 * Revision 1.2  2012-05-15 11:12:10  jbf9160
 * Small updates.
 * MVC Fully working.
 *
 * Revision 1.1  2012-05-15 10:52:05  jbf9160
 * Interface fully working.
 *
 */

/**
 * ATMPhases is the ATM model for the Bank program
 * 
 * @author James Fator (jbf9160@rit.edu)
 */
public class ATMPhases {

	private int phase;
	private Terminal screen;
	private ATMGUI gui;
	private int idNumber;
	private String pinNumberS;
	private int pinNumber;
	
	/**
	 * Constructor for the ATM model
	 * 
	 * @param gui the ATMGUI view for this model
	 * @param screen the screen for the ATMGUI
	 */
	public ATMPhases(ATMGUI gui, Terminal screen) {
		this.gui = gui;
		this.screen = screen;
		this.phase = -1;
		this.idNumber = 0;
		this.pinNumberS = "";
		this.pinNumber = 0;
	}
	
	/**
	 * display is the method that modifies the Terminal output
	 * based on which phase in the process the ATM is in
	 */
	/*Package*/ void display() {
		if (phase == -1) {
			screen.bootSystem();
			phase++;
			this.display();
		} else if (phase == 0) {
			screen.inputPrompt(
					" ENTER ACCOUNT NUMBER", "<CLEAR>", " ");
		} else if (phase == 1) {
			screen.inputPrompt(" ENTER PIN", "<CLEAR>", "<CANCEL>");
		} else if (phase == 2) {
			screen.optionPrompt(" TRANSACTION MENU", "<BALANCE>",
									"<DEPOSIT>", "<WITHDRAW>", "<LOGOUT>");
		} else if (phase == 3) {
			screen.clearScreen();
			screen.slowType(" " + Double.toString(
					gui.getBal(idNumber, pinNumber)));
			screen.slowType(" OKAY TO RETURN");
		} else if (phase == 4) {
			screen.inputPrompt(" ENTER DEPOSIT AMOUNT", "<CANCEL>", " ");
		} else if (phase == 5) {
			screen.inputPrompt(" ENTER WITHDRAW AMOUNT", "<CANCEL>", " ");
		} else if (phase == 6) {
			screen.clearScreen();
			screen.slowType(" SUCCESSFUL DEPOSIT");
			screen.slowType(" OKAY TO RETURN");
		} else if (phase == 7) {
			screen.clearScreen();
			screen.slowType(" INVALID AMOUNT");
			screen.slowType(" OKAY TO RETURN");
		} else if (phase == 8) {
			screen.clearScreen();
			screen.slowType(" SUCCESSFUL WITHDRAW");
			screen.slowType(" OKAY TO RETURN");
		}
	}
	
	/**
	 * okay is the method that is called when the okay button 
	 * has been clicked.
	 * okay will react differently based on which phase of the
	 * ATM process it is in.
	 */
	/*Package*/ void okay() {
		if (phase == 0) {
			int id = this.screen.getEntry();
			if (id == -1) {
				return;
			}
			if (gui.validId(id)) {
				phase++;
				this.idNumber = id;
				this.display();
			} else {
				this.screen.errorCatch(" INVALID ACCOUNT NUMBER");
			}
		} else if (phase == 1) {
			if (!pinNumberS.equals("") && 
					gui.validPin(idNumber, Integer.parseInt(pinNumberS))) {
				phase++;
				this.pinNumber = Integer.parseInt(pinNumberS);
				this.display();
			} else {
				this.screen.errorCatch(" INVALID PIN NUMBER");
				this.pinNumberS = "";
			}
		} else if (phase == 2) {
			
		} else if (phase == 3) {
			this.phase = 2;
			this.display();
		} else if (phase == 4) {
			this.phase = 6;
			double bal = this.screen.getBalEntry();
			gui.deposit(idNumber, pinNumber, bal);
			this.display();
		} else if (phase == 5) {
			double bal = this.screen.getBalEntry();
			if (!gui.withdraw(idNumber, pinNumber, bal)) {
				this.phase = 7;
				this.display();
				return;
			}
			this.phase = 8;
			this.display();
		} else if (phase == 6) {
			this.phase = 2;
			this.display();
		} else if (phase == 7) {
			this.phase = 2;
			this.display();
		} else if (phase == 8) {
			this.phase = 2;
			this.display();
		}
	}
	
	/**
	 * close is the method that is called when the close button
	 * has been hit.
	 * Close will always dispose of the ATMGUI.
	 */
	/*Package*/ void close() {
		this.gui.dispose();
	}
	
	/**
	 * one is the method that is called when the first option
	 * button is clicked. 
	 * one has a variety of options from clearing the screen
	 * to changing the phase of the ATM.
	 */
	/*Package*/ void one() {
		if (phase == 0 || phase == 1) {
			this.screen.clear();
		} else if (phase == 2) {
			this.phase++;
			this.display();
		} else if (phase == 4) {
			this.phase = 2;
			this.display();
		} else if (phase == 5) {
			this.phase = 2;
			this.display();
		}
	}
	
	/**
	 * two is the method that is called when the second option
	 * button is clicked.
	 * two only changes the phases of the ATM depending on which
	 * phase it is currently.
	 */
	/*Package*/ void two() {
		if (phase == 1) {
			this.phase = 0;
			this.display();
		} else if (phase == 2) {
			this.phase = 4;
			this.display();
		}
	}

	/**
	 * three is the method that is called when the third option
	 * button is clicked.
	 * three is only an available option at one point, phase 2
	 * where it changes the model to phase 5
	 */
	/*Package*/ void three() {
		if (phase == 2) {
			this.phase = 5;
			this.display();
		}
	}

	/**
	 * four is the method that is called when the fourth option
	 * 	button is clicked.
	 * four is only available at phase 2 and it's job is to log
	 * 	out and set the ATM to phase 0.
	 */
	/*Package*/ void four() {
		if (phase == 2){
			this.idNumber = 0;
			this.pinNumber = 0;
			phase = 0;
			this.display();
		}
	}
	
	/**
	 * number is the method that is called when one of the
	 * 	keypad buttons are clicked.
	 * Depending on the phase, the number buttons either
	 * 	increment the Terminal with a number or '*' or just
	 * 	do nothing at all.
	 * 
	 * @param num the number of the button that was clicked.
	 */
	/*Package*/ void number(int num) {
		if (phase == 0 || phase == 4 || phase == 5) {
			screen.input(num);
		} else if (phase == 1) {
			this.pinNumberS += Integer.toString(num);
			screen.input('*');
		}
	}
}