/*
 * Account.java
 * 
 * $Id: Account.java,v 1.5 2012-05-16 06:18:11 jbf9160 Exp $
 * 
 * $Log: Account.java,v $
 * Revision 1.5  2012-05-16 06:18:11  jbf9160
 * Comments added.
 *
 * Revision 1.4  2012-05-15 21:09:50  jbf9160
 * BatchMode method created.
 * BatchMode working.
 * Minor updates.
 *
 * Revision 1.3  2012-05-15 06:48:26  jbf9160
 * Added headers... hopefully not to late.
 *
 */

/**
 * Account is the class holding each Account specifications
 * 
 * @author James Fator (jbf9160@rit.edu)
 */
public abstract class Account {

	private char type;
	private int id;
	private int pin;
	private double balance;
	private double minBalance;
	
	/**
	 * Constructor for the Account object
	 * 
	 * @param type 'c' for checking, 's' for saving, 'd' for cd
	 * @param id identification number
	 * @param pin personal identification number
	 * @param balance amount Account holds in bank
	 * @param minBalance minimum balance for specified Account type
	 */
	public Account(char type, int id, int pin, double balance,
											double minBalance) {
		this.type = type;
		this.id = id;
		this.pin = pin;
		this.balance = balance;
		this.minBalance = minBalance;
	}
	
	/**
	 * getType is a getter method for the type variable
	 * 
	 * @return type
	 */
	/*Package*/ char getType() {
		return this.type;
	}
	
	/**
	 * getPin is the getter method for the pin variable
	 * @return pin
	 */
	/*Package*/ int getPin() {
		return this.pin;
	}
	
	/**
	 * getTypeName returns the name of the Account type in capitals
	 * 
	 * @return Account type name
	 */
	/*Package*/ abstract String getTypeName();
	
	/**
	 * getId is the getter method for the id variable
	 * 
	 * @return id
	 */
	/*Package*/ int getId() {
		return this.id;
	}
	
	/**
	 * set id is the setter method for the id variable
	 * 
	 * @param newId id to replace current id with
	 */
	/*Package*/ void setId(int newId) {
		this.id = newId;
	}
	
	/**
	 * getBalance is the getter method for the balance variable
	 * 
	 * @return balance (rounded to two decimal places)
	 */
	/*Package*/ double getBalance() {
		return (double)Math.round(balance * 100) / 100;
	}
	
	/**
	 * setBalance is the setter method for the balance variable
	 * 
	 * @param newBalance new balance for Account
	 */
	/*Package*/ void setBalance(double newBalance) {
		this.balance = newBalance;
	}
	
	/**
	 * withdraw removes money from account.
	 * Does not check if removing too much, that is done by the Bank
	 * 
	 * @param amount the amount to withdraw
	 */
	/*Package*/ void withdraw(double amount) {
		this.balance -= amount;
	}
	
	/**
	 * deposit adds money to the account.
	 * 
	 * @param amount the amount to add
	 */
	/*Package*/ void deposit(double amount) {
		this.balance += amount;
	}
	
	/**
	 * getMinBalance returns the value for the minimum balance for
	 * the specified Account type
	 * 
	 * @return minBalance
	 */
	/*Package*/ double getMinBalance() {
		return this.minBalance;
	}
	
	/**
	 * checkPin tests if the passed integer equals the current pin value
	 * 
	 * @param value integer to test
	 * @return true if the value equals the pin
	 */
	/*Package*/ boolean checkPin(int value) {
		if (this.pin == value) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * phoenix creates a new Account from the destruction of the current
	 * 
	 * @param type new Account char to change to
	 * @return new Account
	 */
	/*Package*/ Account phoenix(char type) {
		if (type == 'd') {
			return new CertificateOfDeposit(
					'd', this.id, this.pin, this.balance);
		} else if (type == 'c') {
			return new Checking(
					'c', this.id, this.pin, this.balance);
		} else {
			return new Saving(
					's', this.id, this.pin, this.balance);
		}
	}
	
	/**
	 * collectInterest adds interest to the account
	 * @return the amount of interest
	 */
	/*Package*/ abstract double collectInterest();
	
	/**
	 * getPenalty removes money from Account if the penalty applies
	 * @return the amount of money removed
	 */
	/*Package*/ abstract double getPenalty();
	
	/**
	 * toString method for the Account class
	 * 
	 * @return id of the Account
	 */
	@Override
	public String toString() {
		return Integer.toString(this.id);
	}
	
	/**
	 * prettyPrint returns the Account type name
	 * 
	 * @return name of Account type
	 */
	public abstract String prettyPrint();
}

/**
 * CertificateOfDeposit is an Account type
 */
class CertificateOfDeposit extends Account {
	
	private double interest;
	
	/**
	 * constructor for the CD Account type
	 * 
	 * @param type 'c'
	 * @param id Account number
	 * @param pin personal identification number
	 * @param balance amount in the Account
	 */
	public CertificateOfDeposit(char type, int id, int pin, double balance) {
		super(type, id, pin, balance, 500);
		this.interest = 0.05/12;
	}
	
	/**
	 * collectInterest adds interest into Account
	 * 
	 * @return the amount added from interest
	 */
	/*Package*/ double collectInterest() {
		double amount = super.getBalance() * this.interest;
		super.deposit(amount);
		return amount;
	}
	
	/**
	 * getTypeName returns "CD"
	 * @return "CD"
	 */
	/*Package*/ String getTypeName() {
		return "CD";
	}
	
	/**
	 * getPenalty returns 0.00
	 * @return 0.00
	 */
	/*Package*/ double getPenalty() {
		return 0.00;
	}
	
	/**
	 * prettyPrint returns "CD      "
	 * @return "CD      "
	 */
	public String prettyPrint() {
		return "CD       ";
	}
}

/**
 * Checking is an Account type
 */
class Checking extends Account {
	
	/**
	 * constructor for Checking Account type
	 * @param type 'c'
	 * @param id Account number
	 * @param pin personal identification number
	 * @param balance amount in the Account
	 */
	public Checking(char type, int id, int pin, double balance) {
		super(type, id, pin, balance, 50);
	}
	
	/**
	 * isPenalty checks if there will be a penalty on the account.
	 * @return true if the balance is less than the minimum balance
	 */
	/*Package*/ boolean isPenalty() {
		return (super.getBalance() < super.getMinBalance());
	}
	
	/**
	 * getPenalty removes the specified penalty amount from the 
	 * Account if the penalty applies
	 * @return the amount taken away
	 */
	/*Package*/ double getPenalty() {
		if (super.getBalance() > super.getMinBalance()) {
			return 0.00;
		}
		if (super.getBalance() > 5) {
			return 5;
		} else {
			return super.getBalance() * 0.10;
		}
	}
	
	/**
	 * getTypeName returns "CHECKING"
	 * @return "CHECKING"
	 */
	/*Package*/ String getTypeName() {
		return "CHECKING";
	}
	
	/**
	 * collectInterest returns 0.00
	 * @return 0.00
	 */
	/*Package*/ double collectInterest() {
		return 0.00;
	}
	
	/**
	 * prettyPrint returns "Checking "
	 * @return "Checking "
	 */
	public String prettyPrint() {
		return "Checking ";
	}
}

/**
 * Saving is an Account type
 */
class Saving extends Account {
	
	private double interest;
	
	/**
	 * constructor for the Saving Account type
	 * @param type 's'
	 * @param id Account number
	 * @param pin personal identification number
	 * @param balance amount in Account
	 */
	public Saving(char type, int id, int pin, double balance) {
		super(type, id, pin, balance, 200);
		this.interest = 0.005/12;
	}
	
	/**
	 * isPenalty checks if a penalty should be applied to Account
	 * @return true if balance is less than the minimum balance
	 */
	/*Package*/ boolean isPenalty() {
		return (super.getBalance() < super.getMinBalance());
	}
	
	/**
	 * getPenalty applied a penalty to an Account
	 * @return the amount of the penalty
	 */
	/*Package*/ double getPenalty() {
		if (super.getBalance() > super.getMinBalance()) {
			return 0.0;
		}
		if (super.getBalance() > 10) {
			return 10;
		} else {
			return super.getBalance() * 0.10;
		}
	}
	
	/**
	 * collectInterest adds interest to the account only if
	 * the balance is more than the minimum balance
	 * @return the amount of the interest collected
	 */
	/*Package*/ double collectInterest() {
		if (super.getBalance() >= super.getMinBalance()) {
			double amount = super.getBalance() * this.interest;
			super.deposit(amount);
			return amount;
		}
		return 0.00;
	}
	
	/**
	 * getTypeName returns "SAVING"
	 * @return "SAVING"
	 */
	/*Package*/ String getTypeName() {
		return "SAVING";
	}
	
	/**
	 * prettyPrint returns "Saving   "
	 * @return "Saving   "
	 */
	public String prettyPrint() {
		return "Saving   ";
	}
}
