/*
 * BankGui.java
 * 
 * $Id: BankGUI.java,v 1.10 2012-05-16 06:18:11 jbf9160 Exp $
 * 
 * $Log: BankGUI.java,v $
 * Revision 1.10  2012-05-16 06:18:11  jbf9160
 * Comments added.
 *
 * Revision 1.9  2012-05-15 21:09:50  jbf9160
 * BatchMode method created.
 * BatchMode working.
 * Minor updates.
 *
 * Revision 1.8  2012-05-15 11:12:11  jbf9160
 * Small updates.
 * MVC Fully working.
 *
 * Revision 1.7  2012-05-15 10:52:05  jbf9160
 * Interface fully working.
 *
 * Revision 1.6  2012-05-15 07:16:27  jbf9160
 * Bank Model and GUI now can test for an account ID.
 *
 * Revision 1.5  2012-05-15 06:48:26  jbf9160
 * Added headers... hopefully not to late.
 *
 */

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Arrays;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * BankGUI is the view and controller for the Bank model
 * 
 * @author James Fator (jbf9160@rit.edu)
 */
public class BankGUI extends JFrame implements Observer {

	private static final long serialVersionUID = 1L;
	private final Bank bank;
	private BankGUI obs = this;
	private JList accountsList;
	private JTextField idField;
	private JTextField typeField;
	private JTextField balanceField;
	private JButton update;
	private JButton ATM;
	private int temp;

	/**
	 * Constructor for the BankGUI class.
	 * Creates window, adds accounts to list,
	 * able to update accounts from window.
	 * 
	 * @param incBank the Bank model
	 */
	public BankGUI(Bank incBank) {
		this.temp = 0;
		this.bank = incBank;
		this.setSize(450, 250);
		this.setTitle("James Fator");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setLocationRelativeTo(null);
		JPanel main = new JPanel(new BorderLayout());
		
		JPanel accountsDisplay = new JPanel(new BorderLayout(10,10));
		accountsDisplay.add(
				new JLabel("      Accounts:      ", JLabel.CENTER),
													BorderLayout.NORTH);
		accountsDisplay.add(new JLabel(" "), BorderLayout.WEST);
		accountsDisplay.add(new JLabel(" "), BorderLayout.EAST);
		accountsList = new JList();
		if (bank.getAccounts().size() == 0) {
			accountsList.setListData(Arrays.asList("NO ACCOUNTS").toArray());
			accountsDisplay.add(accountsList, BorderLayout.CENTER);
		} else {
			accountsList.setListData(bank.getAccounts().toArray());
			JScrollPane accountsScroll = new JScrollPane(accountsList);
			accountsList.addListSelectionListener(
											new ListSelectionListener() {
				//Listens for a new selected item.
				//Updates JTextFields with new information
				public void valueChanged(ListSelectionEvent e) {
					if (accountsList.getSelectedValue() != null) {
						Account account = (Account) accountsList
								.getSelectedValue();
						idField.setText(Integer.toString(account.getId()));
						typeField.setText(account.getTypeName());
						balanceField.setText(Double.toString(account
								.getBalance()));
					} else {
						idField.setText("");
						typeField.setText("");
						balanceField.setText("");
					}
					accountsList.validate();
				}
			});
			accountsDisplay.add(accountsScroll, BorderLayout.CENTER);
		}
		ATM = new JButton("    ATM    ");
		ATM.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				launchATM();
			}
		});
		accountsDisplay.add(new JLabel(" "), BorderLayout.SOUTH);
		//Add accounts to window
		main.add(accountsDisplay, BorderLayout.WEST);
		
		JPanel parameters = new JPanel(new BorderLayout());
		JPanel arguments = new JPanel(new GridLayout(3,2));
		
		idField = new JTextField(13);
		arguments.add(new JLabel("ACCOUNT NUMBER:"));
		JPanel idFieldContainer = new JPanel(new FlowLayout(0,0,14));
		idFieldContainer.add(idField);
		arguments.add(idFieldContainer);
		
		typeField = new JTextField(13);
		arguments.add(new JLabel("ACCOUNT TYPE:"));
		JPanel typeFieldContainer = new JPanel(new FlowLayout(0,0,14));
		typeFieldContainer.add(typeField);
		arguments.add(typeFieldContainer);
		
		balanceField = new JTextField(13);
		arguments.add(new JLabel("BALANCE:"));
		JPanel balFieldContainer = new JPanel(new FlowLayout(0,0,14));
		balFieldContainer.add(balanceField);
		arguments.add(balFieldContainer);
		
		JPanel buttonHolder = new JPanel();
		update = new JButton(" UPDATE ");
		update.addActionListener(new ActionListener() {
			/**
			 * actionPerformed listens for when the update button is
			 * clicked.
			 * Modifies the selected Account based on JTextField input
			 */
			public void actionPerformed(ActionEvent e) {
				if (accountsList.getSelectedValue() == null) {
					return;
				}
				String type = typeField.getText().toUpperCase();
				int err = 0;
				String idText = idField.getText();
				String balText = balanceField.getText();
				if (!"CDCHECKINGSAVING".contains(type)) {
					typeField.setText("INVALID ACCOUNT TYPE");
					err++;
				}
				if (!idText.matches("\\d+") ||
									idText.length() < 4 ||
									idText.length() > 9) {
					idField.setText("INVALID ACCOUNT ID");
					err++;
				}
				if ((balText.contains("E") && balText.substring(
						0,balText.length()-4).matches("-?\\d+(\\.\\d*)?"))) {
					
				} else if (!balText.matches("-?\\d+(\\.\\d*)?")) {
					balanceField.setText("INVALID BALANCE");
					err++;
				}
				if (err == 0) {
					int id = Integer.parseInt(idText);
					double bal;
					if (balText.contains("E")) {
						double pre = Double.parseDouble(
								balText.substring(0, balText.indexOf('E')));
						int exp = Integer.parseInt(balText.substring(
								balText.indexOf('E')+1,balText.length()));
						bal = pre * Math.pow(10, exp);
					} else {
						bal = Double.parseDouble(balText);
					}
					Account account = 
							(Account) accountsList.getSelectedValue();
					account.setBalance(bal);
					account.setId(id);
					if (!type.equals(account.getTypeName())) {
						if (type.equals("CD")) {
							type = "D";
						}
						int index = bank.updateAccount(
								account, type.toLowerCase().charAt(0));
						accountsList.setListData(
								bank.getAccounts().toArray());
						accountsList.setSelectedIndex(index);
					}
					accountsList.updateUI();
					accountsList.validate();
				}
			}
		});
		buttonHolder.add(update);
		buttonHolder.add(ATM);
		
		parameters.add(new JLabel(" Parameters: ", JLabel.CENTER),
				BorderLayout.NORTH);
		parameters.add(arguments, BorderLayout.CENTER);
		parameters.add(buttonHolder, BorderLayout.SOUTH);
		
		this.addWindowListener(new WindowListener() {
			public void windowActivated(WindowEvent arg0) {}
			public void windowClosed(WindowEvent arg0) {}
			/**
			 * windowClosing listens for when the window is about
			 * to close so it can notify the Bank model to 
			 * run the finalData method.
			 */
			@Override
			public void windowClosing(WindowEvent arg0) {
				bank.finalData();
			}
			public void windowDeactivated(WindowEvent arg0) {}
			public void windowDeiconified(WindowEvent arg0) {}
			public void windowIconified(WindowEvent arg0) {}
			public void windowOpened(WindowEvent arg0) {}
		});
		
		main.add(parameters, BorderLayout.CENTER);
		this.add(main);
		this.setResizable(false);
		this.setVisible(true);
	}
	
	private void launchATM() {
		new Thread(new Runnable() {
			/**
			 * run starts a new instance of ATMGUI
			 */
			public void run() {
				temp++;
				new ATMGUI(obs, temp);
			}
		}).start();
	}
	
	/**
	 * validId checks with the Bank model if the id is valid
	 * @param id of Account to check
	 * @return true if Account with id exists
	 */
	/*Package*/ boolean validId(int id) {
		return bank.validId(id);
	}
	
	/**
	 * validPin checks with the Bank model if 
	 * the pin works with the given Account
	 * 
	 * @param id of Account to check
	 * @param pin to test with Account
	 * @return true if pin matches Account
	 * 
	 * @assume id is valid
	 */
	/*Package*/ boolean validPin(int id, int pin) {
		return bank.validPin(id, pin);
	}
	
	/**
	 * getBal retrieves the balance of the account from the
	 * Bank model
	 * 
	 * @param id of Account
	 * @param pin of Account
	 * @return amount in Account
	 * 
	 * @assume id and pin are valid
	 */
	/*Package*/ double getBal(int id, int pin) {
		return bank.getBal(id, pin);
	}
	
	/**
	 * deposit adds money to Account
	 * 
	 * @param id of Account
	 * @param pin of Account
	 * @param bal amount to be added
	 * 
	 * @assume id and pin are valid
	 */
	/*Package*/ void deposit(int id, int pin, double bal) {
		bank.deposit(id, pin, bal);
	}
	
	/**
	 * withdraw removes money from account if the account
	 * has enough
	 * 
	 * @param id of Account
	 * @param pin of Account
	 * @param bal amount to be removed
	 * @return true if withdraw was successful
	 * 
	 * @assume id and pin are valid
	 */
	/*Package*/ boolean withdraw(int id, int pin, double bal) {
		return bank.withdraw(id, pin, bal);
	}

	/**
	 * update is called when the Bank model has changed Account
	 * information. It then updates the accounts list and
	 * displays the changes made.
	 */
	@Override
	public void update(Observable o, Object arg) {
		accountsList.setListData(
				bank.getAccounts().toArray());
		accountsList.updateUI();
		accountsList.validate();
		this.validate();
	}
}