/*
 * Terminal.java
 * 
 * $Id: Terminal.java,v 1.9 2012-05-16 06:18:10 jbf9160 Exp $
 * 
 * $Log: Terminal.java,v $
 * Revision 1.9  2012-05-16 06:18:10  jbf9160
 * Comments added.
 *
 * Revision 1.8  2012-05-15 21:09:50  jbf9160
 * BatchMode method created.
 * BatchMode working.
 * Minor updates.
 *
 * Revision 1.7  2012-05-15 11:12:10  jbf9160
 * Small updates.
 * MVC Fully working.
 *
 * Revision 1.6  2012-05-15 10:52:05  jbf9160
 * Interface fully working.
 *
 * Revision 1.5  2012-05-15 07:16:26  jbf9160
 * Bank Model and GUI now can test for an account ID.
 *
 * Revision 1.4  2012-05-15 06:48:25  jbf9160
 * Added headers... hopefully not to late.
 *
 */

import java.awt.Color;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * Terminal is a view component for the ATMGUI
 * 
 * @author James Fator (jbf9160@rit.edu)
 */
public class Terminal extends JPanel {
	
	private static final long serialVersionUID = 1L;
	private JLabel l1,l2,l3,l4,l5,l6,l7,l8,l9;
	private JLabel command;

	/**
	 * Constructor for the Terminal JPanel
	 * creates a black background with black JLabels that
	 * have green foregrounds which give the command-prompt 
	 * feel
	 */
	public Terminal() {
		this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		l1 = new JLabel(" ");
		l1.setForeground(Color.GREEN.darker());
		l2 = new JLabel(" ");
		l2.setForeground(Color.GREEN.darker());
		l3 = new JLabel(" ");
		l3.setForeground(Color.GREEN.darker());
		l4 = new JLabel(" ");
		l4.setForeground(Color.GREEN.darker());
		l5 = new JLabel(" ");
		l5.setForeground(Color.GREEN.darker());
		l6 = new JLabel(" ");
		l6.setForeground(Color.GREEN.darker());
		l7 = new JLabel(" ");
		l7.setForeground(Color.GREEN.darker());
		l8 = new JLabel(" ");
		l8.setForeground(Color.GREEN.darker());
		l9 = new JLabel(" ");
		l9.setForeground(Color.GREEN.darker());
		command = new JLabel(" ");
		command.setForeground(Color.GREEN.darker());
		this.add(l1);
		this.add(l2);
		this.add(l3);
		this.add(l4);
		this.add(l5);
		this.add(l6);
		this.add(l7);
		this.add(l8);
		this.add(l9);
		this.add(command);
		this.setBackground(Color.BLACK);
	}

	/**
	 * bootSystem is the initial display for the Terminal.
	 * No real significance besides it looks pretty.
	 */
	/*Package*/ void bootSystem() {
		try {
			slowType(" ACME TERMINAL BOOTING UP", 1);
			for (int i = 0; i < 3; i++) {
				Thread.sleep(500);
				input('.');
			}
			Thread.sleep(500);
			slowType(" TERMINAL BOOTED", 1);
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			return;
		}
	}
	
	/**
	 * errorCatch is a method that takes a String input and changes
	 * the display to print the error message
	 * @param text the error message to display
	 */
	/*Package*/ void errorCatch(String text) {
		this.clear();
		this.slowType(text);
		this.generalShiftUpdate();
	}
	
	/**
	 * slowType is the main form of output for the Terminal.
	 * Shifts the current display up and modifies the bottom.
	 * @param text the String to output
	 */
	/*Package*/ void slowType(String text) {
		l6.setText(l7.getText());
		l7.setText(l8.getText());
		l8.setText(l9.getText());
		l9.setText(command.getText());
		command.setText(text);
		command.updateUI();
	}

	/**
	 * slowType that takes an additional argument is the way the 
	 * Terminal can print out a String character by character.
	 * Only used in the bootSystem method however to save time.
	 * 
	 * @param text the String to output character by character
	 * @param go arbitrary integer
	 */
	/*Package*/ synchronized void slowType(final String text, int go) {
		new Thread(new Runnable() {
			@Override
			public void run() {
				l6.setText(l7.getText());
				l7.setText(l8.getText());
				l8.setText(l9.getText());
				l9.setText(command.getText());
				command.setText(" ");
				command.updateUI();
				try {
					for (char c : text.toCharArray()) {
						Thread.sleep(25);
						command.setText(command.getText() + c);
						command.updateUI();
					}
				} catch (InterruptedException ex) {}
			}
		}).start();
		try {
			Thread.sleep(text.toCharArray().length*30);
		} catch (InterruptedException ex) {}
	}
	
	private synchronized void generalShiftUpdate() {
		l6.setText(l7.getText());
		l7.setText(l8.getText());
		l8.setText(l9.getText());
		l9.setText(command.getText());
		command.setText("> ");
		this.validate();
	}
	
	/**
	 * clearScreen sets all the internal JLabels to " " status
	 */
	/*Package*/ synchronized void clearScreen() {
		l1.setText(" ");
		l2.setText(" ");
		l3.setText(" ");
		l4.setText(" ");
		l5.setText(" ");
		l6.setText(" ");
		l7.setText(" ");
		l8.setText(" ");
		l9.setText(" ");
		command.setText(" ");
		this.validate();
	}
	
	/**
	 * enterLine updates a given line with the given text.
	 * 
	 * @param text the String to output
	 * @param line the line to output on
	 */
	/*Package*/ synchronized void enterLine(String text, int line) {
		if (line == 1) {
			l1.setText(text);
		} else if (line == 2) {
			l2.setText(text);
		} else if (line == 3) {
			l3.setText(text);
		} else if (line == 4) {
			l4.setText(text);
		} else if (line == 5) {
			l5.setText(text);
		} else if (line == 6) {
			l6.setText(text);
		} else if (line == 7) {
			l7.setText(text);
		} else if (line == 8) {
			l8.setText(text);
		} else if (line == 9) {
			l9.setText(text);
		} else {
			command.setText(text);
		}
		this.validate();
	}
	
	/**
	 * optionPrompt is the main way to prompt a question with up to
	 * four pre-set choices.
	 * 
	 * @param query the "question" to prompt for
	 * @param o1 the first option
	 * @param o2 the second option
	 * @param o3 the third option
	 * @param o4 the fourth option
	 */
	/*Package*/ synchronized void optionPrompt(
			String query, String o1, String o2, String o3, String o4) {
		for (int i = (55-o1.length()); i>0; i--) {
			o1 = " " + o1;
		}
		for (int j = (55-o2.length()); j>0; j--) {
			o2 = " " + o2;
		}
		for (int u = (55-o3.length()); u>0; u--) {
			o3 = " " + o3;
		}
		for (int k = (55-o4.length()); k>0; k--) {
			o4 = " " + o4;
		}
		this.clearScreen();
		l2.setText(o1);
		l4.setText(o2);
		l6.setText(query);
		l7.setText(o3);
		l9.setText(o4);
		this.validate();
	}
	
	/**
	 * optionLine displays an option at the given location.
	 * @param o1 the option to display
	 * @param line the line which to display at
	 */
	/*Package*/ synchronized void optionLine(String o1, int line) {
		for (int i = (60-o1.length()); i>0; i--) {
			o1 = " " + o1;
		}
		if (line == 2) {
			l2.setText(o1);
		} else if (line == 4) {
			l4.setText(o1);
		} else if (line == 7) {
			l7.setText(o1);
		} else {
			l9.setText(o1);
		}
		this.validate();
	}
	
	/**
	 * input with an integer appends the bottom-most JLabel with 
	 * the given integer.
	 * @param c number to append with
	 */
	/*Package*/ synchronized void input(int c) {
		command.setText(command.getText() + Integer.toString(c));
	}
	
	/**
	 * input with a character appends the bottom-most JLabel with
	 * the given character
	 * @param c the character to append with
	 */
	/*Package*/ synchronized void input(char c) {
		command.setText(command.getText() + Character.toString(c));
	}
	
	/**
	 * delete removes the last character in the bottom-most
	 * JLabel. (Not including the "> ")
	 */
	/*Package*/ synchronized void delete() {
		if (command.getText().length() > 2) {
			command.setText(command.getText().substring(
					0,command.getText().length()-1));
		}
	}
	
	/**
	 * inputPrompt is the main way to ask for input with up to
	 * two options.
	 * 
	 * @param query the "question" to prompt
	 * @param o1 the first option
	 * @param o2 the second option
	 */
	/*Package*/ synchronized void inputPrompt(
			String query, String o1, String o2) {
		this.clearScreen();
		this.slowType(query);
		this.generalShiftUpdate();
		this.optionLine(o1, 2);
		this.optionLine(o2, 4);
	}
	
	/**
	 * clear removes all user input from the bottom-most JLabel.
	 */
	/*Package*/ synchronized void clear() {
		command.setText("> ");
	}
	
	/**
	 * getEntry grabs the user input from the current 
	 * bottom-most JLabel.
	 * 
	 * @return String containing everything in the JLabel at bottom
	 */
	/*Package*/ synchronized int getEntry() {
		this.generalShiftUpdate();
		if (this.l9.getText().length() == 2) {
			return -1;
		}
		return Integer.parseInt(this.l9.getText().substring(2,
				l9.getText().length()));
	}
	
	/**
	 * getBalEntry is the special getEntry that parses the input
	 * for a double value.
	 * @return the balance inputed
	 */
	/*Package*/ synchronized double getBalEntry() {
		this.generalShiftUpdate();
		if (this.l9.getText().length() == 2) {
			return -1;
		}
		return Double.parseDouble(this.l9.getText().substring(2,
				l9.getText().length()));
	}
}
